package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import com.atlassian.elasticsearch.buckler.config.ConfigRestHandler;
import com.atlassian.elasticsearch.buckler.security.AuthRateLimiter;
import com.atlassian.elasticsearch.buckler.security.TimeProvider;
import org.elasticsearch.cluster.metadata.IndexNameExpressionResolver;
import org.elasticsearch.cluster.node.DiscoveryNodes;
import org.elasticsearch.common.inject.Inject;
import org.elasticsearch.common.io.stream.NamedWriteableRegistry;
import org.elasticsearch.common.network.NetworkService;
import org.elasticsearch.common.settings.ClusterSettings;
import org.elasticsearch.common.settings.IndexScopedSettings;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.settings.SettingsFilter;
import org.elasticsearch.common.util.BigArrays;
import org.elasticsearch.common.util.concurrent.ThreadContext;
import org.elasticsearch.common.xcontent.NamedXContentRegistry;
import org.elasticsearch.http.HttpServerTransport;
import org.elasticsearch.indices.breaker.CircuitBreakerService;
import org.elasticsearch.plugins.ActionPlugin;
import org.elasticsearch.plugins.NetworkPlugin;
import org.elasticsearch.plugins.Plugin;
import org.elasticsearch.rest.RestController;
import org.elasticsearch.rest.RestHandler;
import org.elasticsearch.threadpool.ThreadPool;
import org.elasticsearch.transport.Transport;
import org.elasticsearch.transport.TransportInterceptor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;
import java.util.function.UnaryOperator;

/**
 * Buckler security plugin for basic authentication and TLS.
 */
public class BucklerPlugin extends Plugin implements NetworkPlugin, ActionPlugin {

    private final AuthRateLimiter limiter;
    private final BucklerConfig bucklerConfig;

    @Inject
    public BucklerPlugin(Settings settings) throws IOException {
        this.bucklerConfig = new BucklerConfig(settings);
        this.limiter = new AuthRateLimiter(bucklerConfig, new TimeProvider());
    }

    @Override
    public Map<String, Supplier<HttpServerTransport>> getHttpTransports(Settings settings, ThreadPool threadPool,
                                                                        BigArrays bigArrays, CircuitBreakerService circuitBreakerService,
                                                                        NamedWriteableRegistry namedWriteableRegistry,
                                                                        NamedXContentRegistry xContentRegistry,
                                                                        NetworkService networkService, HttpServerTransport.Dispatcher dispatcher) {
        return Collections.singletonMap("http", () -> new SecureHttpServerTransport(settings, networkService, bigArrays,
                bucklerConfig, threadPool, xContentRegistry, dispatcher));
    }

    @Override
    public UnaryOperator<RestHandler> getRestHandlerWrapper(ThreadContext threadContext) {
        return restHandler -> new RestAuthenticationHandler(restHandler, bucklerConfig.getAuthConfig(), limiter);
    }

    @Override
    public List<RestHandler> getRestHandlers(Settings settings, RestController restController,
                                             ClusterSettings clusterSettings, IndexScopedSettings indexScopedSettings,
                                             SettingsFilter settingsFilter, IndexNameExpressionResolver indexNameExpressionResolver,
                                             Supplier<DiscoveryNodes> nodesInCluster) {
        List<RestHandler> arrayList = new ArrayList<>();
        arrayList.add(new ConfigRestHandler(settings, restController, bucklerConfig));
        return arrayList;
    }

    @Override
    public List<TransportInterceptor> getTransportInterceptors(ThreadContext threadContext) {
        return Collections.singletonList(new AuthTransportInterceptor(limiter, bucklerConfig, threadContext));
    }

    @Override
    public Map<String, Supplier<Transport>> getTransports(Settings settings, ThreadPool threadPool, BigArrays bigArrays,
                                                          CircuitBreakerService circuitBreakerService,
                                                          NamedWriteableRegistry namedWriteableRegistry, NetworkService networkService) {
        return Collections.singletonMap("netty_tcp", () -> new SecureTransport(settings, threadPool, networkService, bigArrays, bucklerConfig,
                namedWriteableRegistry, circuitBreakerService));
    }

}
