package com.atlassian.elasticsearch.buckler.config;

import org.apache.logging.log4j.Logger;
import org.elasticsearch.common.io.PathUtils;
import org.elasticsearch.common.logging.ESLoggerFactory;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.xcontent.XContentBuilder;
import org.elasticsearch.common.xcontent.XContentFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.text.MessageFormat;

import static org.elasticsearch.common.Strings.cleanPath;

public class BucklerConfig {

    private static final Logger logger = ESLoggerFactory.getLogger(BucklerConfig.class.getName());

    private final Path configFile;

    private Settings settings;

    // volatile to make sure that new values are visible in case the settings get changed
    private volatile AuthConfig authConfig;
    private volatile BruteForceProtectionSettings bruteForceProtectionSettings;
    private volatile TlsConfig tlsConfig;

    public BucklerConfig(Settings nodeSettings) throws IOException {
        Path configDir = getConfigDir(nodeSettings);
        Path bucklerDir = configDir.resolve("buckler");
        Path config = bucklerDir.resolve("buckler.yml");
        if (Files.exists(config)) {
            try (InputStream inputStream = Files.newInputStream(config)) {
                this.settings = Settings.builder().loadFromStream(config.getFileName().toString(), inputStream).build();
            }
        } else {
            logger.warn("Buckler config file missing, tried path: " + config);
            this.settings = Settings.EMPTY;
        }

        boolean server = nodeSettings.getAsBoolean("network.server", Boolean.TRUE);

        this.configFile = config;

        this.authConfig = new AuthConfig(settings);
        this.tlsConfig = new TlsConfig(settings, server);
        this.bruteForceProtectionSettings = new BruteForceProtectionSettings(settings);
    }

    /**
     * @return the current configuration for authentication; should not be cached because the configuration can change
     */
    public AuthConfig getAuthConfig() {
        return authConfig;
    }

    /**
     * @return the current configuration for security lockout and timeout period
     */
    public BruteForceProtectionSettings getBruteForceProtectionSettings() {
        return bruteForceProtectionSettings;
    }

    /**
     * @return the current configuration for TLS; should not be cached because the configuration can change
     */
    public TlsConfig getTlsConfig() {
        return tlsConfig;
    }

    static String getRequiredSetting(Settings settings, String key, String description) {
        String value = settings.get(key);
        if (value == null || value.trim().isEmpty()) {
            throw new BucklerConfigException(MessageFormat.format(description + " ({0})", key));
        }
        return value;
    }

    static boolean loadEnabled(Settings settings, String enabledKey, String description) {
        boolean enabled = settings.getAsBoolean(enabledKey, false);
        logger.info(MessageFormat.format(description + " enabled ({0}): {1}", enabledKey, enabled));
        return enabled;
    }

    void updateAuthPassword(final String newPassword) {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                Settings updatedSettings = Settings.builder()
                        .put(settings)
                        .put("auth.basic.password", newPassword)
                        .build();
                AuthConfig updatedAuthConfig = new AuthConfig(updatedSettings);

                try (OutputStream outputStream = Files.newOutputStream(configFile);
                     XContentBuilder contentBuilder = XContentFactory.yamlBuilder(outputStream)) {
                    contentBuilder.map(updatedSettings.getAsMap());
                    BucklerConfig.this.settings = updatedSettings;
                    BucklerConfig.this.authConfig = updatedAuthConfig;
                } catch (IOException e) {
                    throw new AuthConfigUpdateException("Failed to write Buckler configuration to the file", e);
                }
                return null;
            }
        });
    }

    // This is done the same way as in Environment. We can't use Environment directly, because it's not available for
    // injection in all the places that we need to work.
    private static Path getConfigDir(Settings settings) {
        if (settings.get("path.conf") != null) {
            return PathUtils.get(cleanPath(settings.get("path.conf")));
        }

        if (settings.get("path.home") != null) {
            Path homeFile = PathUtils.get(cleanPath(settings.get("path.home")));
            return homeFile.resolve("config");
        } else {
            throw new IllegalStateException("path.home is not configured");
        }
    }
}
